@@ -1,4 +1,5 @@
 #include <lader/feature-vector.h>
+#include <boost/foreach.hpp>
 
 using namespace lader;
 
@@ -25,3 +26,11 @@ FeatureVectorInt lader::VectorSubtract(const FeatureVectorInt & a,
     }
     return ret;
 }
+
+void lader::ClearAndSet(FeatureVectorInt & fvi,
+		const IntMap<double> & feat_map)
+{
+    fvi.clear();
+    BOOST_FOREACH(FeaturePairInt feat_pair, feat_map)
+    	fvi.push_back(feat_pair);
+}
