@@ -30,3 +30,11 @@ Ranks::Ranks(const CombinedAlign & combined) {
             ranks_[i] = max_rank_;
     }
 }
+
+void Ranks::SetRanks(const std::vector<int> & order) {
+	if(order.size() != ranks_.size())
+		THROW_ERROR("Vector sizes in Reorder don't match: " << order.size() << " != " << ranks_.size() << endl)
+	// overwrite the new order as ranks
+	ranks_ = order;
+	max_rank_ = order.size()-1;
+}
